/****************************************************************************
** Meta object code from reading C++ file 'qpcsccard_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/nfc/pcsc/qpcsccard_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpcsccard_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QPcscCardE_t {};
} // unnamed namespace

template <> constexpr inline auto QPcscCard::qt_create_metaobjectdata<qt_meta_tag_ZN9QPcscCardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPcscCard",
        "disconnected",
        "",
        "invalidated",
        "requestCompleted",
        "QNearFieldTarget::RequestId",
        "request",
        "QNearFieldTarget::Error",
        "reason",
        "QVariant",
        "result",
        "ndefMessageRead",
        "QNdefMessage",
        "message",
        "onDisconnectRequest",
        "onTargetDestroyed",
        "onSendCommandRequest",
        "command",
        "onReadNdefMessagesRequest",
        "onWriteNdefMessagesRequest",
        "QList<QNdefMessage>",
        "messages",
        "onKeepAliveTimeout",
        "enableAutodelete"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'invalidated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCompleted'
        QtMocHelpers::SignalData<void(const QNearFieldTarget::RequestId &, QNearFieldTarget::Error, const QVariant &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'ndefMessageRead'
        QtMocHelpers::SignalData<void(const QNdefMessage &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onDisconnectRequest'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTargetDestroyed'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSendCommandRequest'
        QtMocHelpers::SlotData<void(const QNearFieldTarget::RequestId &, const QByteArray &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QByteArray, 17 },
        }}),
        // Slot 'onReadNdefMessagesRequest'
        QtMocHelpers::SlotData<void(const QNearFieldTarget::RequestId &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'onWriteNdefMessagesRequest'
        QtMocHelpers::SlotData<void(const QNearFieldTarget::RequestId &, const QList<QNdefMessage> &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onKeepAliveTimeout'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'enableAutodelete'
        QtMocHelpers::MethodData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPcscCard, qt_meta_tag_ZN9QPcscCardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPcscCard::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPcscCardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPcscCardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QPcscCardE_t>.metaTypes,
    nullptr
} };

void QPcscCard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPcscCard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->disconnected(); break;
        case 1: _t->invalidated(); break;
        case 2: _t->requestCompleted((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 3: _t->ndefMessageRead((*reinterpret_cast<std::add_pointer_t<QNdefMessage>>(_a[1]))); break;
        case 4: _t->onDisconnectRequest(); break;
        case 5: _t->onTargetDestroyed(); break;
        case 6: _t->onSendCommandRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 7: _t->onReadNdefMessagesRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1]))); break;
        case 8: _t->onWriteNdefMessagesRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QNdefMessage>>>(_a[2]))); break;
        case 9: _t->onKeepAliveTimeout(); break;
        case 10: _t->enableAutodelete(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNdefMessage >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QNdefMessage> >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPcscCard::*)()>(_a, &QPcscCard::disconnected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPcscCard::*)()>(_a, &QPcscCard::invalidated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPcscCard::*)(const QNearFieldTarget::RequestId & , QNearFieldTarget::Error , const QVariant & )>(_a, &QPcscCard::requestCompleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPcscCard::*)(const QNdefMessage & )>(_a, &QPcscCard::ndefMessageRead, 3))
            return;
    }
}

const QMetaObject *QPcscCard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPcscCard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QPcscCardE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QPcscCard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void QPcscCard::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QPcscCard::invalidated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QPcscCard::requestCompleted(const QNearFieldTarget::RequestId & _t1, QNearFieldTarget::Error _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QPcscCard::ndefMessageRead(const QNdefMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
