/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothlocaldevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qbluetoothlocaldevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothlocaldevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21QBluetoothLocalDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto QBluetoothLocalDevice::qt_create_metaobjectdata<qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBluetoothLocalDevice",
        "hostModeStateChanged",
        "",
        "QBluetoothLocalDevice::HostMode",
        "state",
        "deviceConnected",
        "QBluetoothAddress",
        "address",
        "deviceDisconnected",
        "pairingFinished",
        "QBluetoothLocalDevice::Pairing",
        "pairing",
        "errorOccurred",
        "QBluetoothLocalDevice::Error",
        "error",
        "Pairing",
        "Unpaired",
        "Paired",
        "AuthorizedPaired",
        "HostMode",
        "HostPoweredOff",
        "HostConnectable",
        "HostDiscoverable",
        "HostDiscoverableLimitedInquiry",
        "Error",
        "NoError",
        "PairingError",
        "MissingPermissionsError",
        "UnknownError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hostModeStateChanged'
        QtMocHelpers::SignalData<void(QBluetoothLocalDevice::HostMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'deviceConnected'
        QtMocHelpers::SignalData<void(const QBluetoothAddress &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'deviceDisconnected'
        QtMocHelpers::SignalData<void(const QBluetoothAddress &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'pairingFinished'
        QtMocHelpers::SignalData<void(const QBluetoothAddress &, QBluetoothLocalDevice::Pairing)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QBluetoothLocalDevice::Error)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Pairing'
        QtMocHelpers::EnumData<enum Pairing>(15, 15, QMC::EnumFlags{}).add({
            {   16, Pairing::Unpaired },
            {   17, Pairing::Paired },
            {   18, Pairing::AuthorizedPaired },
        }),
        // enum 'HostMode'
        QtMocHelpers::EnumData<enum HostMode>(19, 19, QMC::EnumFlags{}).add({
            {   20, HostMode::HostPoweredOff },
            {   21, HostMode::HostConnectable },
            {   22, HostMode::HostDiscoverable },
            {   23, HostMode::HostDiscoverableLimitedInquiry },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(24, 24, QMC::EnumFlags{}).add({
            {   25, Error::NoError },
            {   26, Error::PairingError },
            {   27, Error::MissingPermissionsError },
            {   28, Error::UnknownError },
        }),
    };
    return QtMocHelpers::metaObjectData<QBluetoothLocalDevice, qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBluetoothLocalDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>.metaTypes,
    nullptr
} };

void QBluetoothLocalDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBluetoothLocalDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hostModeStateChanged((*reinterpret_cast<std::add_pointer_t<QBluetoothLocalDevice::HostMode>>(_a[1]))); break;
        case 1: _t->deviceConnected((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1]))); break;
        case 2: _t->deviceDisconnected((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1]))); break;
        case 3: _t->pairingFinished((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBluetoothLocalDevice::Pairing>>(_a[2]))); break;
        case 4: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QBluetoothLocalDevice::Error>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothLocalDevice::HostMode >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothLocalDevice::Pairing >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothLocalDevice::Error >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBluetoothLocalDevice::*)(QBluetoothLocalDevice::HostMode )>(_a, &QBluetoothLocalDevice::hostModeStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothLocalDevice::*)(const QBluetoothAddress & )>(_a, &QBluetoothLocalDevice::deviceConnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothLocalDevice::*)(const QBluetoothAddress & )>(_a, &QBluetoothLocalDevice::deviceDisconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothLocalDevice::*)(const QBluetoothAddress & , QBluetoothLocalDevice::Pairing )>(_a, &QBluetoothLocalDevice::pairingFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothLocalDevice::*)(QBluetoothLocalDevice::Error )>(_a, &QBluetoothLocalDevice::errorOccurred, 4))
            return;
    }
}

const QMetaObject *QBluetoothLocalDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothLocalDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QBluetoothLocalDeviceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBluetoothLocalDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QBluetoothLocalDevice::hostModeStateChanged(QBluetoothLocalDevice::HostMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QBluetoothLocalDevice::deviceConnected(const QBluetoothAddress & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QBluetoothLocalDevice::deviceDisconnected(const QBluetoothAddress & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBluetoothLocalDevice::pairingFinished(const QBluetoothAddress & _t1, QBluetoothLocalDevice::Pairing _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QBluetoothLocalDevice::errorOccurred(QBluetoothLocalDevice::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
