/****************************************************************************
** Meta object code from reading C++ file 'qlowenergyservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qlowenergyservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlowenergyservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QLowEnergyServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto QLowEnergyService::qt_create_metaobjectdata<qt_meta_tag_ZN17QLowEnergyServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLowEnergyService",
        "stateChanged",
        "",
        "QLowEnergyService::ServiceState",
        "newState",
        "characteristicChanged",
        "QLowEnergyCharacteristic",
        "info",
        "value",
        "characteristicRead",
        "characteristicWritten",
        "descriptorRead",
        "QLowEnergyDescriptor",
        "descriptorWritten",
        "errorOccurred",
        "QLowEnergyService::ServiceError",
        "error",
        "ServiceType",
        "PrimaryService",
        "IncludedService",
        "ServiceError",
        "NoError",
        "OperationError",
        "CharacteristicWriteError",
        "DescriptorWriteError",
        "UnknownError",
        "CharacteristicReadError",
        "DescriptorReadError",
        "ServiceState",
        "InvalidService",
        "RemoteService",
        "RemoteServiceDiscovering",
        "RemoteServiceDiscovered",
        "LocalService",
        "DiscoveryRequired",
        "DiscoveringService",
        "ServiceDiscovered",
        "DiscoveryMode",
        "FullDiscovery",
        "SkipValueDiscovery",
        "WriteMode",
        "WriteWithResponse",
        "WriteWithoutResponse",
        "WriteSigned"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QLowEnergyService::ServiceState)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'characteristicChanged'
        QtMocHelpers::SignalData<void(const QLowEnergyCharacteristic &, const QByteArray &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'characteristicRead'
        QtMocHelpers::SignalData<void(const QLowEnergyCharacteristic &, const QByteArray &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'characteristicWritten'
        QtMocHelpers::SignalData<void(const QLowEnergyCharacteristic &, const QByteArray &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'descriptorRead'
        QtMocHelpers::SignalData<void(const QLowEnergyDescriptor &, const QByteArray &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'descriptorWritten'
        QtMocHelpers::SignalData<void(const QLowEnergyDescriptor &, const QByteArray &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 7 }, { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QLowEnergyService::ServiceError)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ServiceType'
        QtMocHelpers::EnumData<enum ServiceType>(17, 17, QMC::EnumFlags{}).add({
            {   18, ServiceType::PrimaryService },
            {   19, ServiceType::IncludedService },
        }),
        // enum 'ServiceError'
        QtMocHelpers::EnumData<enum ServiceError>(20, 20, QMC::EnumFlags{}).add({
            {   21, ServiceError::NoError },
            {   22, ServiceError::OperationError },
            {   23, ServiceError::CharacteristicWriteError },
            {   24, ServiceError::DescriptorWriteError },
            {   25, ServiceError::UnknownError },
            {   26, ServiceError::CharacteristicReadError },
            {   27, ServiceError::DescriptorReadError },
        }),
        // enum 'ServiceState'
        QtMocHelpers::EnumData<enum ServiceState>(28, 28, QMC::EnumFlags{}).add({
            {   29, ServiceState::InvalidService },
            {   30, ServiceState::RemoteService },
            {   31, ServiceState::RemoteServiceDiscovering },
            {   32, ServiceState::RemoteServiceDiscovered },
            {   33, ServiceState::LocalService },
            {   34, ServiceState::DiscoveryRequired },
            {   35, ServiceState::DiscoveringService },
            {   36, ServiceState::ServiceDiscovered },
        }),
        // enum 'DiscoveryMode'
        QtMocHelpers::EnumData<enum DiscoveryMode>(37, 37, QMC::EnumFlags{}).add({
            {   38, DiscoveryMode::FullDiscovery },
            {   39, DiscoveryMode::SkipValueDiscovery },
        }),
        // enum 'WriteMode'
        QtMocHelpers::EnumData<enum WriteMode>(40, 40, QMC::EnumFlags{}).add({
            {   41, WriteMode::WriteWithResponse },
            {   42, WriteMode::WriteWithoutResponse },
            {   43, WriteMode::WriteSigned },
        }),
    };
    return QtMocHelpers::metaObjectData<QLowEnergyService, qt_meta_tag_ZN17QLowEnergyServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLowEnergyService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QLowEnergyServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QLowEnergyServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QLowEnergyServiceE_t>.metaTypes,
    nullptr
} };

void QLowEnergyService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLowEnergyService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QLowEnergyService::ServiceState>>(_a[1]))); break;
        case 1: _t->characteristicChanged((*reinterpret_cast<std::add_pointer_t<QLowEnergyCharacteristic>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->characteristicRead((*reinterpret_cast<std::add_pointer_t<QLowEnergyCharacteristic>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 3: _t->characteristicWritten((*reinterpret_cast<std::add_pointer_t<QLowEnergyCharacteristic>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 4: _t->descriptorRead((*reinterpret_cast<std::add_pointer_t<QLowEnergyDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 5: _t->descriptorWritten((*reinterpret_cast<std::add_pointer_t<QLowEnergyDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 6: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QLowEnergyService::ServiceError>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyService::ServiceState >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyDescriptor >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyDescriptor >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyService::ServiceError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(QLowEnergyService::ServiceState )>(_a, &QLowEnergyService::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & )>(_a, &QLowEnergyService::characteristicChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & )>(_a, &QLowEnergyService::characteristicRead, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & )>(_a, &QLowEnergyService::characteristicWritten, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(const QLowEnergyDescriptor & , const QByteArray & )>(_a, &QLowEnergyService::descriptorRead, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(const QLowEnergyDescriptor & , const QByteArray & )>(_a, &QLowEnergyService::descriptorWritten, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyService::*)(QLowEnergyService::ServiceError )>(_a, &QLowEnergyService::errorOccurred, 6))
            return;
    }
}

const QMetaObject *QLowEnergyService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLowEnergyService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QLowEnergyServiceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLowEnergyService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QLowEnergyService::stateChanged(QLowEnergyService::ServiceState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QLowEnergyService::characteristicChanged(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QLowEnergyService::characteristicRead(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QLowEnergyService::characteristicWritten(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QLowEnergyService::descriptorRead(const QLowEnergyDescriptor & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QLowEnergyService::descriptorWritten(const QLowEnergyDescriptor & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QLowEnergyService::errorOccurred(QLowEnergyService::ServiceError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
