/****************************************************************************
** Meta object code from reading C++ file 'bluetoothmanagement_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/bluetoothmanagement_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothmanagement_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19BluetoothManagementE_t {};
} // unnamed namespace

template <> constexpr inline auto BluetoothManagement::qt_create_metaobjectdata<qt_meta_tag_ZN19BluetoothManagementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BluetoothManagement",
        "_q_readNotifier",
        "",
        "processRandomAddressFlagInformation",
        "QBluetoothAddress",
        "address",
        "cleanupOldAddressFlags",
        "EventCode",
        "CommandCompleteEvent",
        "CommandStatusEvent",
        "ControllerErrorEvent",
        "IndexAddedEvent",
        "IndexRemovedEvent",
        "NewSettingsEvent",
        "ClassOfDeviceChangedEvent",
        "LocalNameChangedEvent",
        "NewLinkKeyEvent",
        "NewLongTermKeyEvent",
        "DeviceConnectedEvent",
        "DeviceDisconnectedEvent",
        "ConnectFailedEvent",
        "PINCodeRequestEvent",
        "UserConfirmationRequestEvent",
        "UserPasskeyRequestEvent",
        "AuthenticationFailedEvent",
        "DeviceFoundEvent",
        "DiscoveringEvent",
        "DeviceBlockedEvent",
        "DeviceUnblockedEvent",
        "DeviceUnpairedEvent",
        "PasskeyNotifyEvent",
        "NewIdentityResolvingKeyEvent",
        "NewSignatureResolvingKeyEvent",
        "DeviceAddedEvent",
        "DeviceRemovedEvent",
        "NewConnectionParameterEvent",
        "UnconfiguredIndexAddedEvent",
        "UnconfiguredIndexRemovedEvent",
        "NewConfigurationOptionsEvent",
        "ExtendedIndexAddedEvent",
        "ExtendedIndexRemovedEvent",
        "LocalOutOfBandExtendedDataUpdatedEvent",
        "AdvertisingAddedEvent",
        "AdvertisingRemovedEvent",
        "ExtendedControllerInformationChangedEvent",
        "PHYConfigurationChangedEvent",
        "ExperimentalFeatureChangedEvent",
        "DefaultSystemConfigurationChangedEvent",
        "DefaultRuntimeConfigurationChangedEvent",
        "DeviceFlagsChangedEvent",
        "AdvertisementMonitorAddedEvent",
        "AdvertisementMonitorRemovedEvent",
        "ControllerSuspendEvent",
        "ControllerResumeEvent"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot '_q_readNotifier'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'processRandomAddressFlagInformation'
        QtMocHelpers::SlotData<void(const QBluetoothAddress &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'cleanupOldAddressFlags'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EventCode'
        QtMocHelpers::EnumData<enum EventCode>(7, 7, QMC::EnumIsScoped).add({
            {    8, EventCode::CommandCompleteEvent },
            {    9, EventCode::CommandStatusEvent },
            {   10, EventCode::ControllerErrorEvent },
            {   11, EventCode::IndexAddedEvent },
            {   12, EventCode::IndexRemovedEvent },
            {   13, EventCode::NewSettingsEvent },
            {   14, EventCode::ClassOfDeviceChangedEvent },
            {   15, EventCode::LocalNameChangedEvent },
            {   16, EventCode::NewLinkKeyEvent },
            {   17, EventCode::NewLongTermKeyEvent },
            {   18, EventCode::DeviceConnectedEvent },
            {   19, EventCode::DeviceDisconnectedEvent },
            {   20, EventCode::ConnectFailedEvent },
            {   21, EventCode::PINCodeRequestEvent },
            {   22, EventCode::UserConfirmationRequestEvent },
            {   23, EventCode::UserPasskeyRequestEvent },
            {   24, EventCode::AuthenticationFailedEvent },
            {   25, EventCode::DeviceFoundEvent },
            {   26, EventCode::DiscoveringEvent },
            {   27, EventCode::DeviceBlockedEvent },
            {   28, EventCode::DeviceUnblockedEvent },
            {   29, EventCode::DeviceUnpairedEvent },
            {   30, EventCode::PasskeyNotifyEvent },
            {   31, EventCode::NewIdentityResolvingKeyEvent },
            {   32, EventCode::NewSignatureResolvingKeyEvent },
            {   33, EventCode::DeviceAddedEvent },
            {   34, EventCode::DeviceRemovedEvent },
            {   35, EventCode::NewConnectionParameterEvent },
            {   36, EventCode::UnconfiguredIndexAddedEvent },
            {   37, EventCode::UnconfiguredIndexRemovedEvent },
            {   38, EventCode::NewConfigurationOptionsEvent },
            {   39, EventCode::ExtendedIndexAddedEvent },
            {   40, EventCode::ExtendedIndexRemovedEvent },
            {   41, EventCode::LocalOutOfBandExtendedDataUpdatedEvent },
            {   42, EventCode::AdvertisingAddedEvent },
            {   43, EventCode::AdvertisingRemovedEvent },
            {   44, EventCode::ExtendedControllerInformationChangedEvent },
            {   45, EventCode::PHYConfigurationChangedEvent },
            {   46, EventCode::ExperimentalFeatureChangedEvent },
            {   47, EventCode::DefaultSystemConfigurationChangedEvent },
            {   48, EventCode::DefaultRuntimeConfigurationChangedEvent },
            {   49, EventCode::DeviceFlagsChangedEvent },
            {   50, EventCode::AdvertisementMonitorAddedEvent },
            {   51, EventCode::AdvertisementMonitorRemovedEvent },
            {   52, EventCode::ControllerSuspendEvent },
            {   53, EventCode::ControllerResumeEvent },
        }),
    };
    return QtMocHelpers::metaObjectData<BluetoothManagement, qt_meta_tag_ZN19BluetoothManagementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BluetoothManagement::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BluetoothManagementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BluetoothManagementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19BluetoothManagementE_t>.metaTypes,
    nullptr
} };

void BluetoothManagement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BluetoothManagement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->_q_readNotifier(); break;
        case 1: _t->processRandomAddressFlagInformation((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1]))); break;
        case 2: _t->cleanupOldAddressFlags(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        }
    }
}

const QMetaObject *BluetoothManagement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BluetoothManagement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BluetoothManagementE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int BluetoothManagement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
