/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.dbus.objectmanager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "objectmanageradaptor_p.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

namespace QtBluetoothPrivate {

/*
 * Implementation of adaptor class OrgFreedesktopDBusObjectManagerAdaptor
 */

OrgFreedesktopDBusObjectManagerAdaptor::OrgFreedesktopDBusObjectManagerAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

OrgFreedesktopDBusObjectManagerAdaptor::~OrgFreedesktopDBusObjectManagerAdaptor()
{
    // destructor
}

ManagedObjectList OrgFreedesktopDBusObjectManagerAdaptor::GetManagedObjects()
{
    // handle method call org.freedesktop.DBus.ObjectManager.GetManagedObjects
    ManagedObjectList object_paths_interfaces_and_properties;
    QMetaObject::invokeMethod(parent(), "GetManagedObjects", Q_RETURN_ARG(ManagedObjectList, object_paths_interfaces_and_properties));
    return object_paths_interfaces_and_properties;
}

} // end of namespace QtBluetoothPrivate
