/****************************************************************************
** Meta object code from reading C++ file 'qlowenergycontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qlowenergycontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlowenergycontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QLowEnergyControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto QLowEnergyController::qt_create_metaobjectdata<qt_meta_tag_ZN20QLowEnergyControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLowEnergyController",
        "connected",
        "",
        "disconnected",
        "stateChanged",
        "QLowEnergyController::ControllerState",
        "state",
        "errorOccurred",
        "QLowEnergyController::Error",
        "newError",
        "mtuChanged",
        "mtu",
        "rssiRead",
        "rssi",
        "serviceDiscovered",
        "QBluetoothUuid",
        "newService",
        "discoveryFinished",
        "connectionUpdated",
        "QLowEnergyConnectionParameters",
        "parameters",
        "Error",
        "NoError",
        "UnknownError",
        "UnknownRemoteDeviceError",
        "NetworkError",
        "InvalidBluetoothAdapterError",
        "ConnectionError",
        "AdvertisingError",
        "RemoteHostClosedError",
        "AuthorizationError",
        "MissingPermissionsError",
        "RssiReadError",
        "ControllerState",
        "UnconnectedState",
        "ConnectingState",
        "ConnectedState",
        "DiscoveringState",
        "DiscoveredState",
        "ClosingState",
        "AdvertisingState",
        "RemoteAddressType",
        "PublicAddress",
        "RandomAddress",
        "Role",
        "CentralRole",
        "PeripheralRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QLowEnergyController::ControllerState)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QLowEnergyController::Error)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'mtuChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'rssiRead'
        QtMocHelpers::SignalData<void(qint16)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Short, 13 },
        }}),
        // Signal 'serviceDiscovered'
        QtMocHelpers::SignalData<void(const QBluetoothUuid &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'discoveryFinished'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connectionUpdated'
        QtMocHelpers::SignalData<void(const QLowEnergyConnectionParameters &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(21, 21, QMC::EnumFlags{}).add({
            {   22, Error::NoError },
            {   23, Error::UnknownError },
            {   24, Error::UnknownRemoteDeviceError },
            {   25, Error::NetworkError },
            {   26, Error::InvalidBluetoothAdapterError },
            {   27, Error::ConnectionError },
            {   28, Error::AdvertisingError },
            {   29, Error::RemoteHostClosedError },
            {   30, Error::AuthorizationError },
            {   31, Error::MissingPermissionsError },
            {   32, Error::RssiReadError },
        }),
        // enum 'ControllerState'
        QtMocHelpers::EnumData<enum ControllerState>(33, 33, QMC::EnumFlags{}).add({
            {   34, ControllerState::UnconnectedState },
            {   35, ControllerState::ConnectingState },
            {   36, ControllerState::ConnectedState },
            {   37, ControllerState::DiscoveringState },
            {   38, ControllerState::DiscoveredState },
            {   39, ControllerState::ClosingState },
            {   40, ControllerState::AdvertisingState },
        }),
        // enum 'RemoteAddressType'
        QtMocHelpers::EnumData<enum RemoteAddressType>(41, 41, QMC::EnumFlags{}).add({
            {   42, RemoteAddressType::PublicAddress },
            {   43, RemoteAddressType::RandomAddress },
        }),
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(44, 44, QMC::EnumFlags{}).add({
            {   45, Role::CentralRole },
            {   46, Role::PeripheralRole },
        }),
    };
    return QtMocHelpers::metaObjectData<QLowEnergyController, qt_meta_tag_ZN20QLowEnergyControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLowEnergyController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QLowEnergyControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QLowEnergyControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QLowEnergyControllerE_t>.metaTypes,
    nullptr
} };

void QLowEnergyController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLowEnergyController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QLowEnergyController::ControllerState>>(_a[1]))); break;
        case 3: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QLowEnergyController::Error>>(_a[1]))); break;
        case 4: _t->mtuChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->rssiRead((*reinterpret_cast<std::add_pointer_t<qint16>>(_a[1]))); break;
        case 6: _t->serviceDiscovered((*reinterpret_cast<std::add_pointer_t<QBluetoothUuid>>(_a[1]))); break;
        case 7: _t->discoveryFinished(); break;
        case 8: _t->connectionUpdated((*reinterpret_cast<std::add_pointer_t<QLowEnergyConnectionParameters>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyController::ControllerState >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyController::Error >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothUuid >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QLowEnergyConnectionParameters >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)()>(_a, &QLowEnergyController::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)()>(_a, &QLowEnergyController::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(QLowEnergyController::ControllerState )>(_a, &QLowEnergyController::stateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(QLowEnergyController::Error )>(_a, &QLowEnergyController::errorOccurred, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(int )>(_a, &QLowEnergyController::mtuChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(qint16 )>(_a, &QLowEnergyController::rssiRead, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(const QBluetoothUuid & )>(_a, &QLowEnergyController::serviceDiscovered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)()>(_a, &QLowEnergyController::discoveryFinished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLowEnergyController::*)(const QLowEnergyConnectionParameters & )>(_a, &QLowEnergyController::connectionUpdated, 8))
            return;
    }
}

const QMetaObject *QLowEnergyController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLowEnergyController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QLowEnergyControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLowEnergyController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QLowEnergyController::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QLowEnergyController::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QLowEnergyController::stateChanged(QLowEnergyController::ControllerState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QLowEnergyController::errorOccurred(QLowEnergyController::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QLowEnergyController::mtuChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QLowEnergyController::rssiRead(qint16 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QLowEnergyController::serviceDiscovered(const QBluetoothUuid & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QLowEnergyController::discoveryFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QLowEnergyController::connectionUpdated(const QLowEnergyConnectionParameters & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
