/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothsocket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qbluetoothsocket.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothsocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QBluetoothSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto QBluetoothSocket::qt_create_metaobjectdata<qt_meta_tag_ZN16QBluetoothSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBluetoothSocket",
        "connected",
        "",
        "disconnected",
        "errorOccurred",
        "QBluetoothSocket::SocketError",
        "error",
        "stateChanged",
        "QBluetoothSocket::SocketState",
        "state",
        "serviceDiscovered",
        "QBluetoothServiceInfo",
        "service",
        "discoveryFinished",
        "SocketState",
        "UnconnectedState",
        "ServiceLookupState",
        "ConnectingState",
        "ConnectedState",
        "BoundState",
        "ClosingState",
        "ListeningState",
        "SocketError",
        "NoSocketError",
        "UnknownSocketError",
        "RemoteHostClosedError",
        "HostNotFoundError",
        "ServiceNotFoundError",
        "NetworkError",
        "UnsupportedProtocolError",
        "OperationError",
        "MissingPermissionsError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QBluetoothSocket::SocketError)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QBluetoothSocket::SocketState)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'serviceDiscovered'
        QtMocHelpers::SlotData<void(const QBluetoothServiceInfo &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'discoveryFinished'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SocketState'
        QtMocHelpers::EnumData<enum SocketState>(14, 14, QMC::EnumIsScoped).add({
            {   15, SocketState::UnconnectedState },
            {   16, SocketState::ServiceLookupState },
            {   17, SocketState::ConnectingState },
            {   18, SocketState::ConnectedState },
            {   19, SocketState::BoundState },
            {   20, SocketState::ClosingState },
            {   21, SocketState::ListeningState },
        }),
        // enum 'SocketError'
        QtMocHelpers::EnumData<enum SocketError>(22, 22, QMC::EnumIsScoped).add({
            {   23, SocketError::NoSocketError },
            {   24, SocketError::UnknownSocketError },
            {   25, SocketError::RemoteHostClosedError },
            {   26, SocketError::HostNotFoundError },
            {   27, SocketError::ServiceNotFoundError },
            {   28, SocketError::NetworkError },
            {   29, SocketError::UnsupportedProtocolError },
            {   30, SocketError::OperationError },
            {   31, SocketError::MissingPermissionsError },
        }),
    };
    return QtMocHelpers::metaObjectData<QBluetoothSocket, qt_meta_tag_ZN16QBluetoothSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBluetoothSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QBluetoothSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QBluetoothSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QBluetoothSocketE_t>.metaTypes,
    nullptr
} };

void QBluetoothSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBluetoothSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QBluetoothSocket::SocketError>>(_a[1]))); break;
        case 3: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QBluetoothSocket::SocketState>>(_a[1]))); break;
        case 4: _t->serviceDiscovered((*reinterpret_cast<std::add_pointer_t<QBluetoothServiceInfo>>(_a[1]))); break;
        case 5: _t->discoveryFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothServiceInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBluetoothSocket::*)()>(_a, &QBluetoothSocket::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothSocket::*)()>(_a, &QBluetoothSocket::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothSocket::*)(QBluetoothSocket::SocketError )>(_a, &QBluetoothSocket::errorOccurred, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothSocket::*)(QBluetoothSocket::SocketState )>(_a, &QBluetoothSocket::stateChanged, 3))
            return;
    }
}

const QMetaObject *QBluetoothSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QBluetoothSocketE_t>.strings))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int QBluetoothSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QBluetoothSocket::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QBluetoothSocket::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QBluetoothSocket::errorOccurred(QBluetoothSocket::SocketError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBluetoothSocket::stateChanged(QBluetoothSocket::SocketState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
