/****************************************************************************
** Meta object code from reading C++ file 'gattcharacteristic1adaptor_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/gattcharacteristic1adaptor_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gattcharacteristic1adaptor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor",
        "D-Bus Interface",
        "org.bluez.GattCharacteristic1",
        "D-Bus Introspection",
        "  <interface name=\"org.bluez.GattCharacteristic1\">\n    <method "
        "name=\"ReadValue\">\n      <arg direction=\"in\" type=\"a{sv}\" na"
        "me=\"options\"/>\n      <annotation value=\"QVariantMap\" name=\"o"
        "rg.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg direction=\"out"
        "\" type=\"ay\" name=\"value\"/>\n    </method>\n    <method name=\""
        "WriteValue\">\n      <arg direction=\"in\" type=\"ay\" name=\"valu"
        "e\"/>\n      <arg direction=\"in\" type=\"a{sv}\" name=\"options\""
        "/>\n      <annotation value=\"QVariantMap\" name=\"org.qtproject.Q"
        "tDBus.QtTypeName.In1\"/>\n    </method>\n    <method name=\"StartN"
        "otify\"/>\n    <method name=\"StopNotify\"/>\n    <property access"
        "=\"read\" type=\"s\" name=\"UUID\"/>\n    <property access=\"read\""
        " type=\"o\" name=\"Service\"/>\n    <property access=\"read\" type"
        "=\"ay\" name=\"Value\"/>\n    <property access=\"read\" type=\"b\""
        " name=\"Notifying\"/>\n    <property access=\"read\" type=\"as\" n"
        "ame=\"Flags\"/>\n  </interface>\n",
        "ReadValue",
        "",
        "QVariantMap",
        "options",
        "QDBusMessage",
        "msg",
        "StartNotify",
        "StopNotify",
        "WriteValue",
        "value",
        "Flags",
        "Notifying",
        "Service",
        "QDBusObjectPath",
        "UUID",
        "Value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ReadValue'
        QtMocHelpers::SlotData<QByteArray(const QVariantMap &, const QDBusMessage &)>(5, 6, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'StartNotify'
        QtMocHelpers::SlotData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'StopNotify'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'WriteValue'
        QtMocHelpers::SlotData<void(const QByteArray &, const QVariantMap &, const QDBusMessage &)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 14 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Flags'
        QtMocHelpers::PropertyData<QStringList>(15, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'Notifying'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Service'
        QtMocHelpers::PropertyData<QDBusObjectPath>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'UUID'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Value'
        QtMocHelpers::PropertyData<QByteArray>(20, QMetaType::QByteArray, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OrgBluezGattCharacteristic1Adaptor, qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>.metaTypes,
    nullptr
} };

void QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgBluezGattCharacteristic1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QByteArray _r = _t->ReadValue((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->StartNotify(); break;
        case 2: _t->StopNotify(); break;
        case 3: _t->WriteValue((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->flags(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->notifying(); break;
        case 2: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->service(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->uUID(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->value(); break;
        default: break;
        }
    }
}

const QMetaObject *QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate34OrgBluezGattCharacteristic1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int QtBluetoothPrivate::OrgBluezGattCharacteristic1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
