/****************************************************************************
** Meta object code from reading C++ file 'bluezperipheralobjects_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/bluezperipheralobjects_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluezperipheralobjects_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluezPeripheralGattObject::qt_create_metaobjectdata<qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluezPeripheralGattObject",
        "remoteDeviceAccessEvent",
        "",
        "remoteDeviceObjectPath",
        "mtu"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'remoteDeviceAccessEvent'
        QtMocHelpers::SignalData<void(const QString &, quint16)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::UShort, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtBluezPeripheralGattObject, qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtBluezPeripheralGattObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>.metaTypes,
    nullptr
} };

void QtBluezPeripheralGattObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtBluezPeripheralGattObject *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->remoteDeviceAccessEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint16>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralGattObject::*)(const QString & , quint16 )>(_a, &QtBluezPeripheralGattObject::remoteDeviceAccessEvent, 0))
            return;
    }
}

const QMetaObject *QtBluezPeripheralGattObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluezPeripheralGattObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralGattObjectE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtBluezPeripheralGattObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtBluezPeripheralGattObject::remoteDeviceAccessEvent(const QString & _t1, quint16 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluezPeripheralDescriptor::qt_create_metaobjectdata<qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluezPeripheralDescriptor",
        "valueUpdatedByRemote",
        "",
        "QLowEnergyHandle",
        "characteristicHandle",
        "descriptorHandle",
        "value",
        "ReadValue",
        "QVariantMap",
        "options",
        "QString&",
        "error",
        "WriteValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueUpdatedByRemote'
        QtMocHelpers::SignalData<void(QLowEnergyHandle, QLowEnergyHandle, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 }, { QMetaType::QByteArray, 6 },
        }}),
        // Method 'ReadValue'
        QtMocHelpers::MethodData<QByteArray(const QVariantMap &, QString &)>(7, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Method 'WriteValue'
        QtMocHelpers::MethodData<QString(const QByteArray &, const QVariantMap &)>(12, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 6 }, { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtBluezPeripheralDescriptor, qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtBluezPeripheralDescriptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QtBluezPeripheralGattObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>.metaTypes,
    nullptr
} };

void QtBluezPeripheralDescriptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtBluezPeripheralDescriptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueUpdatedByRemote((*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 1: { QByteArray _r = _t->ReadValue((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->WriteValue((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralDescriptor::*)(QLowEnergyHandle , QLowEnergyHandle , const QByteArray & )>(_a, &QtBluezPeripheralDescriptor::valueUpdatedByRemote, 0))
            return;
    }
}

const QMetaObject *QtBluezPeripheralDescriptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluezPeripheralDescriptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QtBluezPeripheralDescriptorE_t>.strings))
        return static_cast<void*>(this);
    return QtBluezPeripheralGattObject::qt_metacast(_clname);
}

int QtBluezPeripheralDescriptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtBluezPeripheralGattObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QtBluezPeripheralDescriptor::valueUpdatedByRemote(QLowEnergyHandle _t1, QLowEnergyHandle _t2, const QByteArray & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluezPeripheralCharacteristic::qt_create_metaobjectdata<qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluezPeripheralCharacteristic",
        "valueUpdatedByRemote",
        "",
        "QLowEnergyHandle",
        "handle",
        "value",
        "ReadValue",
        "QVariantMap",
        "options",
        "QString&",
        "error",
        "WriteValue",
        "StartNotify",
        "StopNotify"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueUpdatedByRemote'
        QtMocHelpers::SignalData<void(QLowEnergyHandle, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Method 'ReadValue'
        QtMocHelpers::MethodData<QByteArray(const QVariantMap &, QString &)>(6, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Method 'WriteValue'
        QtMocHelpers::MethodData<QString(const QByteArray &, const QVariantMap &)>(11, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 5 }, { 0x80000000 | 7, 8 },
        }}),
        // Method 'StartNotify'
        QtMocHelpers::MethodData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'StopNotify'
        QtMocHelpers::MethodData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtBluezPeripheralCharacteristic, qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtBluezPeripheralCharacteristic::staticMetaObject = { {
    QMetaObject::SuperData::link<QtBluezPeripheralGattObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>.metaTypes,
    nullptr
} };

void QtBluezPeripheralCharacteristic::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtBluezPeripheralCharacteristic *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueUpdatedByRemote((*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 1: { QByteArray _r = _t->ReadValue((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->WriteValue((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->StartNotify(); break;
        case 4: _t->StopNotify(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralCharacteristic::*)(QLowEnergyHandle , const QByteArray & )>(_a, &QtBluezPeripheralCharacteristic::valueUpdatedByRemote, 0))
            return;
    }
}

const QMetaObject *QtBluezPeripheralCharacteristic::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluezPeripheralCharacteristic::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QtBluezPeripheralCharacteristicE_t>.strings))
        return static_cast<void*>(this);
    return QtBluezPeripheralGattObject::qt_metacast(_clname);
}

int QtBluezPeripheralCharacteristic::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtBluezPeripheralGattObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QtBluezPeripheralCharacteristic::valueUpdatedByRemote(QLowEnergyHandle _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN24QtBluezPeripheralServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluezPeripheralService::qt_create_metaobjectdata<qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluezPeripheralService"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtBluezPeripheralService, qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtBluezPeripheralService::staticMetaObject = { {
    QMetaObject::SuperData::link<QtBluezPeripheralGattObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>.metaTypes,
    nullptr
} };

void QtBluezPeripheralService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtBluezPeripheralService *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QtBluezPeripheralService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluezPeripheralService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QtBluezPeripheralServiceE_t>.strings))
        return static_cast<void*>(this);
    return QtBluezPeripheralGattObject::qt_metacast(_clname);
}

int QtBluezPeripheralService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtBluezPeripheralGattObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
