/****************************************************************************
** Meta object code from reading C++ file 'objectmanageradaptor_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/objectmanageradaptor_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'objectmanageradaptor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor",
        "D-Bus Interface",
        "org.freedesktop.DBus.ObjectManager",
        "D-Bus Introspection",
        "  <interface name=\"org.freedesktop.DBus.ObjectManager\">\n    <me"
        "thod name=\"GetManagedObjects\">\n      <arg direction=\"out\" typ"
        "e=\"a{oa{sa{sv}}}\" name=\"object_paths_interfaces_and_properties\""
        "/>\n      <annotation value=\"ManagedObjectList\" name=\"org.qtpro"
        "ject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n    <signal name=\""
        "InterfacesAdded\">\n      <arg type=\"o\" name=\"object_path\"/>\n"
        "      <arg type=\"a{sa{sv}}\" name=\"interfaces_and_properties\"/>"
        "\n      <annotation value=\"InterfaceList\" name=\"org.qtproject.Q"
        "tDBus.QtTypeName.Out1\"/>\n    </signal>\n    <signal name=\"Inter"
        "facesRemoved\">\n      <arg type=\"o\" name=\"object_path\"/>\n   "
        "   <arg type=\"as\" name=\"interfaces\"/>\n    </signal>\n  </inte"
        "rface>\n",
        "InterfacesAdded",
        "",
        "QDBusObjectPath",
        "object_path",
        "InterfaceList",
        "interfaces_and_properties",
        "InterfacesRemoved",
        "interfaces",
        "GetManagedObjects",
        "ManagedObjectList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'InterfacesAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, InterfaceList)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'InterfacesRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QStringList &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::QStringList, 12 },
        }}),
        // Slot 'GetManagedObjects'
        QtMocHelpers::SlotData<ManagedObjectList()>(13, 6, QMC::AccessPublic, 0x80000000 | 14),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OrgFreedesktopDBusObjectManagerAdaptor, qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>.metaTypes,
    nullptr
} };

void QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopDBusObjectManagerAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->InterfacesAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<InterfaceList>>(_a[2]))); break;
        case 1: _t->InterfacesRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: { ManagedObjectList _r = _t->GetManagedObjects();
            if (_a[0]) *reinterpret_cast<ManagedObjectList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< InterfaceList >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopDBusObjectManagerAdaptor::*)(const QDBusObjectPath & , InterfaceList )>(_a, &OrgFreedesktopDBusObjectManagerAdaptor::InterfacesAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopDBusObjectManagerAdaptor::*)(const QDBusObjectPath & , const QStringList & )>(_a, &OrgFreedesktopDBusObjectManagerAdaptor::InterfacesRemoved, 1))
            return;
    }
}

const QMetaObject *QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate38OrgFreedesktopDBusObjectManagerAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::InterfacesAdded(const QDBusObjectPath & _t1, InterfaceList _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QtBluetoothPrivate::OrgFreedesktopDBusObjectManagerAdaptor::InterfacesRemoved(const QDBusObjectPath & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
