/****************************************************************************
** Meta object code from reading C++ file 'qbluetooth.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qbluetooth.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetooth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QBluetoothE_t {};
} // unnamed namespace

template <> constexpr inline auto QBluetooth::qt_create_metaobjectdata<qt_meta_tag_ZN10QBluetoothE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBluetooth",
        "Security",
        "NoSecurity",
        "Authorization",
        "Authentication",
        "Encryption",
        "Secure",
        "AttAccessConstraint",
        "AttAuthorizationRequired",
        "AttAuthenticationRequired",
        "AttEncryptionRequired"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Security'
        QtMocHelpers::EnumData<Security>(1, 1, QMC::EnumIsScoped).add({
            {    2, Security::NoSecurity },
            {    3, Security::Authorization },
            {    4, Security::Authentication },
            {    5, Security::Encryption },
            {    6, Security::Secure },
        }),
        // enum 'AttAccessConstraint'
        QtMocHelpers::EnumData<AttAccessConstraint>(7, 7, QMC::EnumIsScoped).add({
            {    8, AttAccessConstraint::AttAuthorizationRequired },
            {    9, AttAccessConstraint::AttAuthenticationRequired },
            {   10, AttAccessConstraint::AttEncryptionRequired },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10QBluetoothE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10QBluetoothE =
    QBluetooth::qt_create_metaobjectdata<qt_meta_tag_ZN10QBluetoothE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10QBluetoothE =
    qt_staticMetaObjectContent_ZN10QBluetoothE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10QBluetoothE =
    qt_staticMetaObjectContent_ZN10QBluetoothE.relocatingData;

Q_CONSTINIT const QMetaObject QBluetooth::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10QBluetoothE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10QBluetoothE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10QBluetoothE.metaTypes,
    nullptr
} };

QT_WARNING_POP
