/****************************************************************************
** Meta object code from reading C++ file 'qnearfieldtarget_pcsc_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/nfc/qnearfieldtarget_pcsc_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qnearfieldtarget_pcsc_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t {};
} // unnamed namespace

template <> constexpr inline auto QNearFieldTargetPrivateImpl::qt_create_metaobjectdata<qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QNearFieldTargetPrivateImpl",
        "disconnectRequest",
        "",
        "sendCommandRequest",
        "QNearFieldTarget::RequestId",
        "request",
        "command",
        "readNdefMessagesRequest",
        "writeNdefMessagesRequest",
        "QList<QNdefMessage>",
        "messages",
        "targetLost",
        "QNearFieldTargetPrivate*",
        "target",
        "onDisconnected",
        "onInvalidated",
        "onRequestCompleted",
        "QNearFieldTarget::Error",
        "reason",
        "QVariant",
        "result",
        "onNdefMessageRead",
        "QNdefMessage",
        "message"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'disconnectRequest'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendCommandRequest'
        QtMocHelpers::SignalData<void(const QNearFieldTarget::RequestId &, const QByteArray &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QByteArray, 6 },
        }}),
        // Signal 'readNdefMessagesRequest'
        QtMocHelpers::SignalData<void(const QNearFieldTarget::RequestId &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'writeNdefMessagesRequest'
        QtMocHelpers::SignalData<void(const QNearFieldTarget::RequestId &, const QList<QNdefMessage> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'targetLost'
        QtMocHelpers::SignalData<void(QNearFieldTargetPrivate *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onDisconnected'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onInvalidated'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRequestCompleted'
        QtMocHelpers::SlotData<void(const QNearFieldTarget::RequestId &, QNearFieldTarget::Error, const QVariant &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'onNdefMessageRead'
        QtMocHelpers::SlotData<void(const QNdefMessage &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QNearFieldTargetPrivateImpl, qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QNearFieldTargetPrivateImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QNearFieldTargetPrivate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>.metaTypes,
    nullptr
} };

void QNearFieldTargetPrivateImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QNearFieldTargetPrivateImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->disconnectRequest(); break;
        case 1: _t->sendCommandRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->readNdefMessagesRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1]))); break;
        case 3: _t->writeNdefMessagesRequest((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QNdefMessage>>>(_a[2]))); break;
        case 4: _t->targetLost((*reinterpret_cast<std::add_pointer_t<QNearFieldTargetPrivate*>>(_a[1]))); break;
        case 5: _t->onDisconnected(); break;
        case 6: _t->onInvalidated(); break;
        case 7: _t->onRequestCompleted((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 8: _t->onNdefMessageRead((*reinterpret_cast<std::add_pointer_t<QNdefMessage>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTargetPrivate* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTargetPrivateImpl::*)()>(_a, &QNearFieldTargetPrivateImpl::disconnectRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTargetPrivateImpl::*)(const QNearFieldTarget::RequestId & , const QByteArray & )>(_a, &QNearFieldTargetPrivateImpl::sendCommandRequest, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTargetPrivateImpl::*)(const QNearFieldTarget::RequestId & )>(_a, &QNearFieldTargetPrivateImpl::readNdefMessagesRequest, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTargetPrivateImpl::*)(const QNearFieldTarget::RequestId & , const QList<QNdefMessage> & )>(_a, &QNearFieldTargetPrivateImpl::writeNdefMessagesRequest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTargetPrivateImpl::*)(QNearFieldTargetPrivate * )>(_a, &QNearFieldTargetPrivateImpl::targetLost, 4))
            return;
    }
}

const QMetaObject *QNearFieldTargetPrivateImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QNearFieldTargetPrivateImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QNearFieldTargetPrivateImplE_t>.strings))
        return static_cast<void*>(this);
    return QNearFieldTargetPrivate::qt_metacast(_clname);
}

int QNearFieldTargetPrivateImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QNearFieldTargetPrivate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QNearFieldTargetPrivateImpl::disconnectRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QNearFieldTargetPrivateImpl::sendCommandRequest(const QNearFieldTarget::RequestId & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QNearFieldTargetPrivateImpl::readNdefMessagesRequest(const QNearFieldTarget::RequestId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QNearFieldTargetPrivateImpl::writeNdefMessagesRequest(const QNearFieldTarget::RequestId & _t1, const QList<QNdefMessage> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QNearFieldTargetPrivateImpl::targetLost(QNearFieldTargetPrivate * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
