/****************************************************************************
** Meta object code from reading C++ file 'bluezperipheralapplication_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/bluezperipheralapplication_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluezperipheralapplication_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluezPeripheralApplication::qt_create_metaobjectdata<qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluezPeripheralApplication",
        "errorOccurred",
        "",
        "registered",
        "remoteDeviceAccessEvent",
        "remoteDeviceObjectPath",
        "mtu",
        "characteristicValueUpdatedByRemote",
        "QLowEnergyHandle",
        "handle",
        "value",
        "descriptorValueUpdatedByRemote",
        "characteristicHandle",
        "descriptorHandle",
        "GetManagedObjects",
        "ManagedObjectList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'registered'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remoteDeviceAccessEvent'
        QtMocHelpers::SignalData<void(const QString &, quint16)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::UShort, 6 },
        }}),
        // Signal 'characteristicValueUpdatedByRemote'
        QtMocHelpers::SignalData<void(QLowEnergyHandle, const QByteArray &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QByteArray, 10 },
        }}),
        // Signal 'descriptorValueUpdatedByRemote'
        QtMocHelpers::SignalData<void(QLowEnergyHandle, QLowEnergyHandle, const QByteArray &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 12 }, { 0x80000000 | 8, 13 }, { QMetaType::QByteArray, 10 },
        }}),
        // Method 'GetManagedObjects'
        QtMocHelpers::MethodData<ManagedObjectList()>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtBluezPeripheralApplication, qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtBluezPeripheralApplication::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>.metaTypes,
    nullptr
} };

void QtBluezPeripheralApplication::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtBluezPeripheralApplication *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorOccurred(); break;
        case 1: _t->registered(); break;
        case 2: _t->remoteDeviceAccessEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint16>>(_a[2]))); break;
        case 3: _t->characteristicValueUpdatedByRemote((*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 4: _t->descriptorValueUpdatedByRemote((*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QLowEnergyHandle>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 5: { ManagedObjectList _r = _t->GetManagedObjects();
            if (_a[0]) *reinterpret_cast<ManagedObjectList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralApplication::*)()>(_a, &QtBluezPeripheralApplication::errorOccurred, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralApplication::*)()>(_a, &QtBluezPeripheralApplication::registered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralApplication::*)(const QString & , quint16 )>(_a, &QtBluezPeripheralApplication::remoteDeviceAccessEvent, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralApplication::*)(QLowEnergyHandle , const QByteArray & )>(_a, &QtBluezPeripheralApplication::characteristicValueUpdatedByRemote, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtBluezPeripheralApplication::*)(QLowEnergyHandle , QLowEnergyHandle , const QByteArray & )>(_a, &QtBluezPeripheralApplication::descriptorValueUpdatedByRemote, 4))
            return;
    }
}

const QMetaObject *QtBluezPeripheralApplication::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluezPeripheralApplication::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QtBluezPeripheralApplicationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtBluezPeripheralApplication::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QtBluezPeripheralApplication::errorOccurred()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtBluezPeripheralApplication::registered()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtBluezPeripheralApplication::remoteDeviceAccessEvent(const QString & _t1, quint16 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QtBluezPeripheralApplication::characteristicValueUpdatedByRemote(QLowEnergyHandle _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QtBluezPeripheralApplication::descriptorValueUpdatedByRemote(QLowEnergyHandle _t1, QLowEnergyHandle _t2, const QByteArray & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
