/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothdevicediscoveryagent.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/qbluetoothdevicediscoveryagent.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothdevicediscoveryagent.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t {};
} // unnamed namespace

template <> constexpr inline auto QBluetoothDeviceDiscoveryAgent::qt_create_metaobjectdata<qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBluetoothDeviceDiscoveryAgent",
        "deviceDiscovered",
        "",
        "QBluetoothDeviceInfo",
        "info",
        "deviceUpdated",
        "QBluetoothDeviceInfo::Fields",
        "updatedFields",
        "finished",
        "errorOccurred",
        "QBluetoothDeviceDiscoveryAgent::Error",
        "error",
        "canceled",
        "start",
        "DiscoveryMethods",
        "method",
        "stop",
        "Error",
        "NoError",
        "InputOutputError",
        "PoweredOffError",
        "InvalidBluetoothAdapterError",
        "UnsupportedPlatformError",
        "UnsupportedDiscoveryMethod",
        "LocationServiceTurnedOffError",
        "MissingPermissionsError",
        "UnknownError",
        "DiscoveryMethod",
        "NoMethod",
        "ClassicMethod",
        "LowEnergyMethod"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deviceDiscovered'
        QtMocHelpers::SignalData<void(const QBluetoothDeviceInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'deviceUpdated'
        QtMocHelpers::SignalData<void(const QBluetoothDeviceInfo &, QBluetoothDeviceInfo::Fields)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QBluetoothDeviceDiscoveryAgent::Error)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<void(DiscoveryMethods)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(17, 17, QMC::EnumFlags{}).add({
            {   18, Error::NoError },
            {   19, Error::InputOutputError },
            {   20, Error::PoweredOffError },
            {   21, Error::InvalidBluetoothAdapterError },
            {   22, Error::UnsupportedPlatformError },
            {   23, Error::UnsupportedDiscoveryMethod },
            {   24, Error::LocationServiceTurnedOffError },
            {   25, Error::MissingPermissionsError },
            {   26, Error::UnknownError },
        }),
        // flag 'DiscoveryMethods'
        QtMocHelpers::EnumData<DiscoveryMethods>(14, 27, QMC::EnumIsFlag).add({
            {   28, DiscoveryMethod::NoMethod },
            {   29, DiscoveryMethod::ClassicMethod },
            {   30, DiscoveryMethod::LowEnergyMethod },
        }),
    };
    return QtMocHelpers::metaObjectData<QBluetoothDeviceDiscoveryAgent, qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBluetoothDeviceDiscoveryAgent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>.metaTypes,
    nullptr
} };

void QBluetoothDeviceDiscoveryAgent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBluetoothDeviceDiscoveryAgent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deviceDiscovered((*reinterpret_cast<std::add_pointer_t<QBluetoothDeviceInfo>>(_a[1]))); break;
        case 1: _t->deviceUpdated((*reinterpret_cast<std::add_pointer_t<QBluetoothDeviceInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBluetoothDeviceInfo::Fields>>(_a[2]))); break;
        case 2: _t->finished(); break;
        case 3: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QBluetoothDeviceDiscoveryAgent::Error>>(_a[1]))); break;
        case 4: _t->canceled(); break;
        case 5: _t->start(); break;
        case 6: _t->start((*reinterpret_cast<std::add_pointer_t<DiscoveryMethods>>(_a[1]))); break;
        case 7: _t->stop(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothDeviceInfo >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothDeviceInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBluetoothDeviceDiscoveryAgent::*)(const QBluetoothDeviceInfo & )>(_a, &QBluetoothDeviceDiscoveryAgent::deviceDiscovered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothDeviceDiscoveryAgent::*)(const QBluetoothDeviceInfo & , QBluetoothDeviceInfo::Fields )>(_a, &QBluetoothDeviceDiscoveryAgent::deviceUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothDeviceDiscoveryAgent::*)()>(_a, &QBluetoothDeviceDiscoveryAgent::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothDeviceDiscoveryAgent::*)(QBluetoothDeviceDiscoveryAgent::Error )>(_a, &QBluetoothDeviceDiscoveryAgent::errorOccurred, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBluetoothDeviceDiscoveryAgent::*)()>(_a, &QBluetoothDeviceDiscoveryAgent::canceled, 4))
            return;
    }
}

const QMetaObject *QBluetoothDeviceDiscoveryAgent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothDeviceDiscoveryAgent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QBluetoothDeviceDiscoveryAgentE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBluetoothDeviceDiscoveryAgent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QBluetoothDeviceDiscoveryAgent::deviceDiscovered(const QBluetoothDeviceInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QBluetoothDeviceDiscoveryAgent::deviceUpdated(const QBluetoothDeviceInfo & _t1, QBluetoothDeviceInfo::Fields _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QBluetoothDeviceDiscoveryAgent::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QBluetoothDeviceDiscoveryAgent::errorOccurred(QBluetoothDeviceDiscoveryAgent::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QBluetoothDeviceDiscoveryAgent::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
