/****************************************************************************
** Meta object code from reading C++ file 'profile1context_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/profile1context_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'profile1context_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgBluezProfile1ContextInterface::qt_create_metaobjectdata<qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgBluezProfile1ContextInterface",
        "D-Bus Interface",
        "org.bluez.Profile1",
        "D-Bus Introspection",
        "  <interface name=\"org.bluez.Profile1\">\n    <method name=\"NewC"
        "onnection\">\n      <arg direction=\"in\" type=\"o\"/>\n      <arg"
        " direction=\"in\" type=\"h\"/>\n      <arg direction=\"in\" type=\""
        "a{sv}\"/>\n      <annotation value=\"QVariantMap\" name=\"org.qtpr"
        "oject.QtDBus.QtTypeName.In2\"/>\n    </method>\n    <method name=\""
        "RequestDisconnection\">\n      <arg direction=\"in\" type=\"o\"/>\n"
        "    </method>\n    <method name=\"Release\">\n      <annotation va"
        "lue=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n    <"
        "/method>\n  </interface>\n",
        "newConnection",
        "",
        "QDBusUnixFileDescriptor",
        "fd",
        "NewConnection",
        "QDBusObjectPath",
        "QVariantMap",
        "RequestDisconnection",
        "Release",
        "Q_NOREPLY"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newConnection'
        QtMocHelpers::SignalData<void(const QDBusUnixFileDescriptor &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'NewConnection'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &, const QDBusUnixFileDescriptor &, const QVariantMap &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 6 }, { 0x80000000 | 7, 6 }, { 0x80000000 | 11, 6 },
        }}),
        // Slot 'RequestDisconnection'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 6 },
        }}),
        // Slot 'Release'
        QtMocHelpers::SlotData<void()>(13, 14, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OrgBluezProfile1ContextInterface, qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OrgBluezProfile1ContextInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgBluezProfile1ContextInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgBluezProfile1ContextInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newConnection((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1]))); break;
        case 1: _t->NewConnection((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3]))); break;
        case 2: _t->RequestDisconnection((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 3: _t->Release(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusUnixFileDescriptor >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusUnixFileDescriptor >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgBluezProfile1ContextInterface::*)(const QDBusUnixFileDescriptor & )>(_a, &OrgBluezProfile1ContextInterface::newConnection, 0))
            return;
    }
}

const QMetaObject *OrgBluezProfile1ContextInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezProfile1ContextInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgBluezProfile1ContextInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int OrgBluezProfile1ContextInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OrgBluezProfile1ContextInterface::newConnection(const QDBusUnixFileDescriptor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
