/****************************************************************************
** Meta object code from reading C++ file 'bluez_data_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/bluez_data_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluez_data_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QBluezConstE_t {};
} // unnamed namespace

template <> constexpr inline auto QBluezConst::qt_create_metaobjectdata<qt_meta_tag_ZN11QBluezConstE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBluezConst",
        "OpCodeGroupField",
        "OgfLinkControl",
        "OpCodeCommandField",
        "OcfLeSetAdvParams",
        "OcfLeReadTxPowerLevel",
        "OcfLeSetAdvData",
        "OcfLeSetScanResponseData",
        "OcfLeSetAdvEnable",
        "OcfLeClearWhiteList",
        "OcfLeAddToWhiteList",
        "OcfLeConnectionUpdate",
        "AttCommand",
        "ATT_OP_ERROR_RESPONSE",
        "ATT_OP_EXCHANGE_MTU_REQUEST",
        "ATT_OP_EXCHANGE_MTU_RESPONSE",
        "ATT_OP_FIND_INFORMATION_REQUEST",
        "ATT_OP_FIND_INFORMATION_RESPONSE",
        "ATT_OP_FIND_BY_TYPE_VALUE_REQUEST",
        "ATT_OP_FIND_BY_TYPE_VALUE_RESPONSE",
        "ATT_OP_READ_BY_TYPE_REQUEST",
        "ATT_OP_READ_BY_TYPE_RESPONSE",
        "ATT_OP_READ_REQUEST",
        "ATT_OP_READ_RESPONSE",
        "ATT_OP_READ_BLOB_REQUEST",
        "ATT_OP_READ_BLOB_RESPONSE",
        "ATT_OP_READ_MULTIPLE_REQUEST",
        "ATT_OP_READ_MULTIPLE_RESPONSE",
        "ATT_OP_READ_BY_GROUP_REQUEST",
        "ATT_OP_READ_BY_GROUP_RESPONSE",
        "ATT_OP_WRITE_REQUEST",
        "ATT_OP_WRITE_RESPONSE",
        "ATT_OP_PREPARE_WRITE_REQUEST",
        "ATT_OP_PREPARE_WRITE_RESPONSE",
        "ATT_OP_EXECUTE_WRITE_REQUEST",
        "ATT_OP_EXECUTE_WRITE_RESPONSE",
        "ATT_OP_HANDLE_VAL_NOTIFICATION",
        "ATT_OP_HANDLE_VAL_INDICATION",
        "ATT_OP_HANDLE_VAL_CONFIRMATION",
        "ATT_OP_WRITE_COMMAND",
        "ATT_OP_SIGNED_WRITE_COMMAND",
        "AttError",
        "ATT_ERROR_NO_ERROR",
        "ATT_ERROR_INVALID_HANDLE",
        "ATT_ERROR_READ_NOT_PERM",
        "ATT_ERROR_WRITE_NOT_PERM",
        "ATT_ERROR_INVALID_PDU",
        "ATT_ERROR_INSUF_AUTHENTICATION",
        "ATT_ERROR_REQUEST_NOT_SUPPORTED",
        "ATT_ERROR_INVALID_OFFSET",
        "ATT_ERROR_INSUF_AUTHORIZATION",
        "ATT_ERROR_PREPARE_QUEUE_FULL",
        "ATT_ERROR_ATTRIBUTE_NOT_FOUND",
        "ATT_ERROR_ATTRIBUTE_NOT_LONG",
        "ATT_ERROR_INSUF_ENCR_KEY_SIZE",
        "ATT_ERROR_INVAL_ATTR_VALUE_LEN",
        "ATT_ERROR_UNLIKELY",
        "ATT_ERROR_INSUF_ENCRYPTION",
        "ATT_ERROR_UNSUPPRTED_GROUP_TYPE",
        "ATT_ERROR_INSUF_RESOURCES",
        "ATT_ERROR_APPLICATION_START",
        "ATT_ERROR_REQUEST_STALLED",
        "ATT_ERROR_APPLICATION_END"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'OpCodeGroupField'
        QtMocHelpers::EnumData<OpCodeGroupField>(1, 1, QMC::EnumFlags{}).add({
            {    2, OpCodeGroupField::OgfLinkControl },
        }),
        // enum 'OpCodeCommandField'
        QtMocHelpers::EnumData<OpCodeCommandField>(3, 3, QMC::EnumFlags{}).add({
            {    4, OpCodeCommandField::OcfLeSetAdvParams },
            {    5, OpCodeCommandField::OcfLeReadTxPowerLevel },
            {    6, OpCodeCommandField::OcfLeSetAdvData },
            {    7, OpCodeCommandField::OcfLeSetScanResponseData },
            {    8, OpCodeCommandField::OcfLeSetAdvEnable },
            {    9, OpCodeCommandField::OcfLeClearWhiteList },
            {   10, OpCodeCommandField::OcfLeAddToWhiteList },
            {   11, OpCodeCommandField::OcfLeConnectionUpdate },
        }),
        // enum 'AttCommand'
        QtMocHelpers::EnumData<AttCommand>(12, 12, QMC::EnumIsScoped).add({
            {   13, AttCommand::ATT_OP_ERROR_RESPONSE },
            {   14, AttCommand::ATT_OP_EXCHANGE_MTU_REQUEST },
            {   15, AttCommand::ATT_OP_EXCHANGE_MTU_RESPONSE },
            {   16, AttCommand::ATT_OP_FIND_INFORMATION_REQUEST },
            {   17, AttCommand::ATT_OP_FIND_INFORMATION_RESPONSE },
            {   18, AttCommand::ATT_OP_FIND_BY_TYPE_VALUE_REQUEST },
            {   19, AttCommand::ATT_OP_FIND_BY_TYPE_VALUE_RESPONSE },
            {   20, AttCommand::ATT_OP_READ_BY_TYPE_REQUEST },
            {   21, AttCommand::ATT_OP_READ_BY_TYPE_RESPONSE },
            {   22, AttCommand::ATT_OP_READ_REQUEST },
            {   23, AttCommand::ATT_OP_READ_RESPONSE },
            {   24, AttCommand::ATT_OP_READ_BLOB_REQUEST },
            {   25, AttCommand::ATT_OP_READ_BLOB_RESPONSE },
            {   26, AttCommand::ATT_OP_READ_MULTIPLE_REQUEST },
            {   27, AttCommand::ATT_OP_READ_MULTIPLE_RESPONSE },
            {   28, AttCommand::ATT_OP_READ_BY_GROUP_REQUEST },
            {   29, AttCommand::ATT_OP_READ_BY_GROUP_RESPONSE },
            {   30, AttCommand::ATT_OP_WRITE_REQUEST },
            {   31, AttCommand::ATT_OP_WRITE_RESPONSE },
            {   32, AttCommand::ATT_OP_PREPARE_WRITE_REQUEST },
            {   33, AttCommand::ATT_OP_PREPARE_WRITE_RESPONSE },
            {   34, AttCommand::ATT_OP_EXECUTE_WRITE_REQUEST },
            {   35, AttCommand::ATT_OP_EXECUTE_WRITE_RESPONSE },
            {   36, AttCommand::ATT_OP_HANDLE_VAL_NOTIFICATION },
            {   37, AttCommand::ATT_OP_HANDLE_VAL_INDICATION },
            {   38, AttCommand::ATT_OP_HANDLE_VAL_CONFIRMATION },
            {   39, AttCommand::ATT_OP_WRITE_COMMAND },
            {   40, AttCommand::ATT_OP_SIGNED_WRITE_COMMAND },
        }),
        // enum 'AttError'
        QtMocHelpers::EnumData<AttError>(41, 41, QMC::EnumIsScoped).add({
            {   42, AttError::ATT_ERROR_NO_ERROR },
            {   43, AttError::ATT_ERROR_INVALID_HANDLE },
            {   44, AttError::ATT_ERROR_READ_NOT_PERM },
            {   45, AttError::ATT_ERROR_WRITE_NOT_PERM },
            {   46, AttError::ATT_ERROR_INVALID_PDU },
            {   47, AttError::ATT_ERROR_INSUF_AUTHENTICATION },
            {   48, AttError::ATT_ERROR_REQUEST_NOT_SUPPORTED },
            {   49, AttError::ATT_ERROR_INVALID_OFFSET },
            {   50, AttError::ATT_ERROR_INSUF_AUTHORIZATION },
            {   51, AttError::ATT_ERROR_PREPARE_QUEUE_FULL },
            {   52, AttError::ATT_ERROR_ATTRIBUTE_NOT_FOUND },
            {   53, AttError::ATT_ERROR_ATTRIBUTE_NOT_LONG },
            {   54, AttError::ATT_ERROR_INSUF_ENCR_KEY_SIZE },
            {   55, AttError::ATT_ERROR_INVAL_ATTR_VALUE_LEN },
            {   56, AttError::ATT_ERROR_UNLIKELY },
            {   57, AttError::ATT_ERROR_INSUF_ENCRYPTION },
            {   58, AttError::ATT_ERROR_UNSUPPRTED_GROUP_TYPE },
            {   59, AttError::ATT_ERROR_INSUF_RESOURCES },
            {   60, AttError::ATT_ERROR_APPLICATION_START },
            {   61, AttError::ATT_ERROR_REQUEST_STALLED },
            {   62, AttError::ATT_ERROR_APPLICATION_END },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN11QBluezConstE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN11QBluezConstE =
    QBluezConst::qt_create_metaobjectdata<qt_meta_tag_ZN11QBluezConstE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN11QBluezConstE =
    qt_staticMetaObjectContent_ZN11QBluezConstE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN11QBluezConstE =
    qt_staticMetaObjectContent_ZN11QBluezConstE.relocatingData;

Q_CONSTINIT const QMetaObject QBluezConst::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN11QBluezConstE.stringdata,
    qt_staticMetaObjectStaticContent_ZN11QBluezConstE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN11QBluezConstE.metaTypes,
    nullptr
} };

QT_WARNING_POP
