/****************************************************************************
** Meta object code from reading C++ file 'qnearfieldtarget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/nfc/qnearfieldtarget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qnearfieldtarget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QNearFieldTargetE_t {};
} // unnamed namespace

template <> constexpr inline auto QNearFieldTarget::qt_create_metaobjectdata<qt_meta_tag_ZN16QNearFieldTargetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QNearFieldTarget",
        "disconnected",
        "",
        "ndefMessageRead",
        "QNdefMessage",
        "message",
        "requestCompleted",
        "QNearFieldTarget::RequestId",
        "id",
        "error",
        "QNearFieldTarget::Error",
        "Type",
        "ProprietaryTag",
        "NfcTagType1",
        "NfcTagType2",
        "NfcTagType3",
        "NfcTagType4",
        "NfcTagType4A",
        "NfcTagType4B",
        "MifareTag",
        "AccessMethod",
        "UnknownAccess",
        "NdefAccess",
        "TagTypeSpecificAccess",
        "AnyAccess",
        "Error",
        "NoError",
        "UnknownError",
        "UnsupportedError",
        "TargetOutOfRangeError",
        "NoResponseError",
        "ChecksumMismatchError",
        "InvalidParametersError",
        "ConnectionError",
        "NdefReadError",
        "NdefWriteError",
        "CommandError",
        "TimeoutError",
        "UnsupportedTargetError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ndefMessageRead'
        QtMocHelpers::SignalData<void(const QNdefMessage &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'requestCompleted'
        QtMocHelpers::SignalData<void(const QNearFieldTarget::RequestId &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QNearFieldTarget::Error, const QNearFieldTarget::RequestId &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 9 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(11, 11, QMC::EnumFlags{}).add({
            {   12, Type::ProprietaryTag },
            {   13, Type::NfcTagType1 },
            {   14, Type::NfcTagType2 },
            {   15, Type::NfcTagType3 },
            {   16, Type::NfcTagType4 },
            {   17, Type::NfcTagType4A },
            {   18, Type::NfcTagType4B },
            {   19, Type::MifareTag },
        }),
        // enum 'AccessMethod'
        QtMocHelpers::EnumData<enum AccessMethod>(20, 20, QMC::EnumFlags{}).add({
            {   21, AccessMethod::UnknownAccess },
            {   22, AccessMethod::NdefAccess },
            {   23, AccessMethod::TagTypeSpecificAccess },
            {   24, AccessMethod::AnyAccess },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(25, 25, QMC::EnumFlags{}).add({
            {   26, Error::NoError },
            {   27, Error::UnknownError },
            {   28, Error::UnsupportedError },
            {   29, Error::TargetOutOfRangeError },
            {   30, Error::NoResponseError },
            {   31, Error::ChecksumMismatchError },
            {   32, Error::InvalidParametersError },
            {   33, Error::ConnectionError },
            {   34, Error::NdefReadError },
            {   35, Error::NdefWriteError },
            {   36, Error::CommandError },
            {   37, Error::TimeoutError },
            {   38, Error::UnsupportedTargetError },
        }),
    };
    return QtMocHelpers::metaObjectData<QNearFieldTarget, qt_meta_tag_ZN16QNearFieldTargetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QNearFieldTarget::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QNearFieldTargetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QNearFieldTargetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QNearFieldTargetE_t>.metaTypes,
    nullptr
} };

void QNearFieldTarget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QNearFieldTarget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->disconnected(); break;
        case 1: _t->ndefMessageRead((*reinterpret_cast<std::add_pointer_t<QNdefMessage>>(_a[1]))); break;
        case 2: _t->requestCompleted((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[1]))); break;
        case 3: _t->error((*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::Error>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QNearFieldTarget::RequestId>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNearFieldTarget::RequestId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTarget::*)()>(_a, &QNearFieldTarget::disconnected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTarget::*)(const QNdefMessage & )>(_a, &QNearFieldTarget::ndefMessageRead, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTarget::*)(const QNearFieldTarget::RequestId & )>(_a, &QNearFieldTarget::requestCompleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QNearFieldTarget::*)(QNearFieldTarget::Error , const QNearFieldTarget::RequestId & )>(_a, &QNearFieldTarget::error, 3))
            return;
    }
}

const QMetaObject *QNearFieldTarget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QNearFieldTarget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QNearFieldTargetE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QNearFieldTarget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QNearFieldTarget::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QNearFieldTarget::ndefMessageRead(const QNdefMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QNearFieldTarget::requestCompleted(const QNearFieldTarget::RequestId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QNearFieldTarget::error(QNearFieldTarget::Error _t1, const QNearFieldTarget::RequestId & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
