/****************************************************************************
** Meta object code from reading C++ file 'leadvertisement1adaptor_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtconnectivity/src/bluetooth/bluez/leadvertisement1adaptor_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'leadvertisement1adaptor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor",
        "D-Bus Interface",
        "org.bluez.LEAdvertisement1",
        "D-Bus Introspection",
        "  <interface name=\"org.bluez.LEAdvertisement1\">\n    <method nam"
        "e=\"Release\">\n      <annotation value=\"true\" name=\"org.freede"
        "sktop.DBus.Method.NoReply\"/>\n    </method>\n    <property access"
        "=\"readwrite\" type=\"s\" name=\"Type\"/>\n    <property access=\""
        "readwrite\" type=\"as\" name=\"ServiceUUIDs\"/>\n    <property acc"
        "ess=\"readwrite\" type=\"a{qv}\" name=\"ManufacturerData\">\n     "
        " <annotation value=\"ManufacturerDataList\" name=\"org.qtproject.Q"
        "tDBus.QtTypeName\"/>\n    </property>\n    <property access=\"read"
        "write\" type=\"b\" name=\"Discoverable\"/>\n    <property access=\""
        "readwrite\" type=\"as\" name=\"Includes\"/>\n    <property access="
        "\"readwrite\" type=\"s\" name=\"LocalName\"/>\n    <property acces"
        "s=\"readwrite\" type=\"u\" name=\"MinInterval\"/>\n    <property a"
        "ccess=\"readwrite\" type=\"u\" name=\"MaxInterval\"/>\n  </interfa"
        "ce>\n",
        "Release",
        "Q_NOREPLY",
        "Discoverable",
        "Includes",
        "LocalName",
        "ManufacturerData",
        "ManufacturerDataList",
        "MaxInterval",
        "MinInterval",
        "ServiceUUIDs",
        "Type"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'Release'
        QtMocHelpers::SlotData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Discoverable'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Includes'
        QtMocHelpers::PropertyData<QStringList>(8, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'LocalName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'ManufacturerData'
        QtMocHelpers::PropertyData<ManufacturerDataList>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'MaxInterval'
        QtMocHelpers::PropertyData<uint>(12, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'MinInterval'
        QtMocHelpers::PropertyData<uint>(13, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'ServiceUUIDs'
        QtMocHelpers::PropertyData<QStringList>(14, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Type'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OrgBluezLEAdvertisement1Adaptor, qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>.metaTypes,
    nullptr
} };

void QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgBluezLEAdvertisement1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Release(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ManufacturerDataList >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->discoverable(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->includes(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->localName(); break;
        case 3: *reinterpret_cast<ManufacturerDataList*>(_v) = _t->manufacturerData(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->maxInterval(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->minInterval(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->serviceUUIDs(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDiscoverable(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setIncludes(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setLocalName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setManufacturerData(*reinterpret_cast<ManufacturerDataList*>(_v)); break;
        case 4: _t->setMaxInterval(*reinterpret_cast<uint*>(_v)); break;
        case 5: _t->setMinInterval(*reinterpret_cast<uint*>(_v)); break;
        case 6: _t->setServiceUUIDs(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setType(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtBluetoothPrivate31OrgBluezLEAdvertisement1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int QtBluetoothPrivate::OrgBluezLEAdvertisement1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
