/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.bluez.Profile1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef PROFILE1_P_H
#define PROFILE1_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include <QtCore/private/qglobal_p.h>

namespace QtBluetoothPrivate {

/*
 * Proxy class for interface org.bluez.Profile1
 */
class OrgBluezProfile1Interface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.bluez.Profile1"; }

public:
    OrgBluezProfile1Interface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgBluezProfile1Interface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> NewConnection(const QDBusObjectPath &in0, const QDBusUnixFileDescriptor &in1, const QVariantMap &in2)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1) << QVariant::fromValue(in2);
        return asyncCallWithArgumentList(QStringLiteral("NewConnection"), argumentList);
    }

    Q_NOREPLY inline void Release()
    {
        QList<QVariant> argumentList;
        callWithArgumentList(QDBus::NoBlock, QStringLiteral("Release"), argumentList);
    }

    inline QDBusPendingReply<> RequestDisconnection(const QDBusObjectPath &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("RequestDisconnection"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

} // end of namespace QtBluetoothPrivate

namespace org {
  namespace bluez {
    using Profile1 = ::QtBluetoothPrivate::OrgBluezProfile1Interface;
  }
}

#endif
